@@ -1,5 +1,7 @@
 test_that("test_cpv() works correctly", {
 
+  skip_on_cran()
+
   expect_error(test_cpv(c("a", "b", "c"), 1, 0.1), "'dist' must be a numeric vector")
   expect_error(test_cpv(c(0, 1, 2), "a", 0.1), "'teststat' must be a single numeric value")
   expect_error(test_cpv(c(0, 1, 2), c(1.1, 1.2), 0.1), "'teststat' must be a single numeric value")
@@ -22,6 +24,8 @@ test_that("test_cpv() works correctly", {
 
 test_that("simes() works correctly", {
 
+  skip_on_cran()
+
   expect_error(simes(pvals = c("a", "b"), alpha = 0.05), "'pvals' must be a numeric vector")
   expect_error(simes(pvals = c(0.1, 0.05, 1.1), alpha = 0.05), "'pvals' must lie between 0 and 1")
   expect_error(simes(pvals = c(0.3, 0.4), alpha = "a"), "'alpha' must be a single numeric value")
@@ -68,6 +72,8 @@ test_that("simes() works correctly", {
 
 test_that("multi_cutoff() works correctly", {
 
+  skip_on_cran()
+
   library(robust2sls)
   p <- generate_param(1, 1, 1, seed = 40)
   d <- generate_data(parameters = p, n = 1000)$data
@@ -165,6 +171,8 @@ test_that("multi_cutoff() works correctly", {
 
 test_that("proptest() works correctly", {
 
+  skip_on_cran()
+
   p <- generate_param(3, 2, 3, sigma = 2, intercept = TRUE, seed = 42)
   d <- generate_data(parameters = p, n = 1000)$data
   gammas <- seq(0.01, 0.1, 0.01)
@@ -282,10 +290,27 @@ test_that("proptest() works correctly", {
   expect_identical(pval1side, b$pval)
   expect_identical(pval2side, a$pval)
 
+  # single model until convergence (codecov)
+  model <- outlier_detection(data = d, formula = p$setting$formula,
+                             initial_est = "robustified", ref_dist = "normal",
+                             sign_level = 0.01, iterations = "convergence", convergence_criterion = 0)
+  c <- proptest(model, alpha = 0.1, iteration = "convergence", one_sided = FALSE)
+  expect_equal(NROW(c), 1)
+  expect_equal(NCOL(c), 8)
+  expect_equal(c$iter_test, "convergence")
+  expect_equal(c$iter_act, 4)
+  expect_equal(c$gamma, 0.01)
+  expect_equal(c$type, "two-sided")
+  expect_equal(c$alpha, 0.1)
+  expect_equal(c$reject, FALSE)
+  expect_snapshot_output(c)
+
 })
 
 test_that("proptest() raises correct errors", {
 
+  skip_on_cran()
+
   expect_error(proptest(robust2sls_object = 1, alpha = 0.05, iteration = 1,
                         one_sided = TRUE),
                "'robust2sls_object' must be of class 'robust2sls' or a list")
@@ -332,6 +357,8 @@ test_that("proptest() raises correct errors", {
 
 test_that("counttest() raises correct errors", {
 
+  skip_on_cran()
+
   expect_error(counttest(robust2sls_object = 1, alpha = 0.05, iteration = 1,
                         one_sided = TRUE),
                "'robust2sls_object' must be of class 'robust2sls' or a list")
@@ -373,11 +400,25 @@ test_that("counttest() raises correct errors", {
   expect_error(counttest(models, alpha = 0.01, iteration = 1,
                         one_sided = c(TRUE, FALSE)),
                "'one_sided' must be a logical value of length one")
+  expect_error(counttest(models, alpha = 0.01, iteration = 1,
+                         one_sided = TRUE, tsmethod = 1),
+               "must be NULL or a character vector")
+  expect_error(counttest(models, alpha = 0.01, iteration = 1,
+                         one_sided = TRUE, tsmethod = c("a", "b")),
+               "must be of length 1")
+  expect_error(counttest(models, alpha = 0.01, iteration = 1,
+                         one_sided = TRUE, tsmethod = "blabla"),
+               "should be one of \"central\", \"minlike\", \"blaker\"")
 
 })
 
 test_that("counttest() works correctly", {
 
+  skip_on_cran()
+
+  # note: since changed from poisson.test() to poisson.exact() need to specify
+  # tsmethod = "minlike" to reproduce tests as had done originally
+
   p <- generate_param(3, 2, 3, sigma = 2, intercept = TRUE, seed = 42)
   d <- generate_data(parameters = p, n = 1000)$data
   gammas <- seq(0.01, 0.1, 0.01)
@@ -391,10 +432,10 @@ test_that("counttest() works correctly", {
 
   # for smaller gammas seems to converge (quicker) but for larger ones not
   # 0.07, 0.08, 0.1 it goes up to 200 iterations
-  a <- counttest(models, alpha = 0.05, iteration = 0, one_sided = FALSE)
-  b <- counttest(models, alpha = 0.05, iteration = 1, one_sided = FALSE)
+  a <- counttest(models, alpha = 0.05, iteration = 0, one_sided = FALSE, tsmethod = "minlike")
+  b <- counttest(models, alpha = 0.05, iteration = 1, one_sided = FALSE, tsmethod = "minlike")
   c <- counttest(models, alpha = 0.05, iteration = "convergence",
-                one_sided = FALSE)
+                one_sided = FALSE, tsmethod = "minlike")
 
   expect_equal(class(a), "data.frame")
   expect_equal(class(b), "data.frame")
@@ -402,15 +443,18 @@ test_that("counttest() works correctly", {
   expect_equal(NROW(a), length(gammas))
   expect_equal(NROW(b), length(gammas))
   expect_equal(NROW(c), length(gammas))
-  expect_equal(NCOL(a), 9)
-  expect_equal(NCOL(b), 9)
-  expect_equal(NCOL(c), 9)
+  expect_equal(NCOL(a), 10)
+  expect_equal(NCOL(b), 10)
+  expect_equal(NCOL(c), 10)
   expect_equal(colnames(a), c("iter_test", "iter_act", "gamma", "num_act",
-                              "num_exp", "type", "pval", "alpha", "reject"))
+                              "num_exp", "type", "pval", "alpha", "reject",
+                              "tsmethod"))
   expect_equal(colnames(b), c("iter_test", "iter_act", "gamma", "num_act",
-                              "num_exp", "type", "pval", "alpha", "reject"))
+                              "num_exp", "type", "pval", "alpha", "reject",
+                              "tsmethod"))
   expect_equal(colnames(c), c("iter_test", "iter_act", "gamma", "num_act",
-                              "num_exp", "type", "pval", "alpha", "reject"))
+                              "num_exp", "type", "pval", "alpha", "reject",
+                              "tsmethod"))
   expect_equal(a$iter_test, rep(0, 10))
   expect_equal(b$iter_test, rep(1, 10))
   expect_equal(c$iter_test, rep("convergence", 10))
@@ -431,6 +475,9 @@ test_that("counttest() works correctly", {
                            TRUE, FALSE, FALSE))
   expect_equal(c$reject, c(FALSE, FALSE, FALSE, TRUE, TRUE, TRUE, TRUE, TRUE,
                            FALSE, FALSE))
+  expect_equal(attr(a, "tsmethod"), "minlike")
+  expect_equal(attr(b, "tsmethod"), "minlike")
+  expect_equal(attr(c, "tsmethod"), "minlike")
   expect_snapshot_output(a)
   expect_snapshot_output(b)
   expect_snapshot_output(c)
@@ -439,18 +486,20 @@ test_that("counttest() works correctly", {
   models <- multi_cutoff(gamma = gammas, data = d, formula = p$setting$formula,
                          ref_dist = "normal", initial_est = "robustified",
                          iterations = 10)
-  a <- counttest(models, alpha = 0.05, iteration = 3, one_sided = TRUE)
-  b <- counttest(models, alpha = 0.01, iteration = 5, one_sided = FALSE)
+  a <- counttest(models, alpha = 0.05, iteration = 3, one_sided = TRUE, tsmethod = "minlike")
+  b <- counttest(models, alpha = 0.01, iteration = 5, one_sided = FALSE, tsmethod = "minlike")
   expect_equal(class(a), "data.frame")
   expect_equal(class(b), "data.frame")
   expect_equal(NROW(a), length(gammas))
   expect_equal(NROW(b), length(gammas))
-  expect_equal(NCOL(a), 9)
-  expect_equal(NCOL(b), 9)
+  expect_equal(NCOL(a), 10)
+  expect_equal(NCOL(b), 10)
   expect_equal(colnames(a), c("iter_test", "iter_act", "gamma", "num_act",
-                              "num_exp", "type", "pval", "alpha", "reject"))
+                              "num_exp", "type", "pval", "alpha", "reject",
+                              "tsmethod"))
   expect_equal(colnames(b), c("iter_test", "iter_act", "gamma", "num_act",
-                              "num_exp", "type", "pval", "alpha", "reject"))
+                              "num_exp", "type", "pval", "alpha", "reject",
+                              "tsmethod"))
   expect_equal(a$alpha, rep(0.05, 10))
   expect_equal(b$alpha, rep(0.01, 10))
   expect_equal(a$iter_test, rep(3, 10))
@@ -459,6 +508,8 @@ test_that("counttest() works correctly", {
   expect_equal(b$iter_act, rep(5, 10))
   expect_equal(a$type, rep("one-sided", 10))
   expect_equal(b$type, rep("two-sided", 10))
+  expect_equal(attr(a, "tsmethod"), NULL) # no such attribute expected b/c one-sided
+  expect_equal(attr(b, "tsmethod"), "minlike")
 
   expect_snapshot_output(a)
   expect_snapshot_output(b)
@@ -467,16 +518,18 @@ test_that("counttest() works correctly", {
   model <- outlier_detection(data = d, formula = p$setting$formula,
                              initial_est = "saturated", ref_dist = "normal",
                              sign_level = 0.05, iterations = 3, split = 0.5)
-  a <- counttest(model, alpha = 0.1, iteration = 1, one_sided = FALSE)
-  b <- counttest(model, alpha = 0.1, iteration = 1, one_sided = TRUE)
+  a <- counttest(model, alpha = 0.1, iteration = 1, one_sided = FALSE, tsmethod = "minlike")
+  b <- counttest(model, alpha = 0.1, iteration = 1, one_sided = TRUE, tsmethod = "minlike")
   expect_equal(NROW(a), 1)
   expect_equal(NROW(b), 1)
-  expect_equal(NCOL(a), 9)
-  expect_equal(NCOL(b), 9)
+  expect_equal(NCOL(a), 10)
+  expect_equal(NCOL(b), 10)
   expect_equal(colnames(a), c("iter_test", "iter_act", "gamma", "num_act",
-                              "num_exp", "type", "pval", "alpha", "reject"))
+                              "num_exp", "type", "pval", "alpha", "reject",
+                              "tsmethod"))
   expect_equal(colnames(b), c("iter_test", "iter_act", "gamma", "num_act",
-                              "num_exp", "type", "pval", "alpha", "reject"))
+                              "num_exp", "type", "pval", "alpha", "reject",
+                              "tsmethod"))
   expect_equal(a$iter_test, 1)
   expect_equal(b$iter_test, 1)
   expect_equal(a$iter_act, 1)
@@ -491,6 +544,8 @@ test_that("counttest() works correctly", {
   expect_equal(b$alpha, 0.1)
   expect_equal(a$reject, FALSE)
   expect_equal(b$reject, FALSE)
+  expect_equal(attr(a, "tsmethod"), "minlike")
+  expect_equal(attr(b, "tsmethod"), NULL)
   expect_snapshot_output(a)
   expect_snapshot_output(b)
   # re-build p-values manually
@@ -503,10 +558,53 @@ test_that("counttest() works correctly", {
   expect_identical(a$pval, pval2side)
   expect_identical(b$pval, pval1side)
 
+  # try different tsmethods for two-sided
+  a <- counttest(model, alpha = 0.1, iteration = 1, one_sided = FALSE, tsmethod = "central")
+  b <- counttest(model, alpha = 0.1, iteration = 1, one_sided = FALSE, tsmethod = "minlike")
+  c <- counttest(model, alpha = 0.1, iteration = 1, one_sided = FALSE, tsmethod = "blaker")
+  expect_equal(attr(a, "tsmethod"), "central")
+  expect_equal(attr(b, "tsmethod"), "minlike")
+  expect_equal(attr(c, "tsmethod"), "blaker")
+  expect_equal(a$tsmethod, "central")
+  expect_equal(b$tsmethod, "minlike")
+  expect_equal(c$tsmethod, "blaker")
+  expect_snapshot_output(a)
+  expect_snapshot_output(b)
+  expect_snapshot_output(c)
+  # manual computation for the first two (third is hard to compute)
+  num.act <- outliers(model, iteration = 1)
+  num.exp <- model$cons$sign_level * NROW(d)
+  ## central
+  ### one-sided lower
+  less <- ppois(q = num.act, lambda = num.exp, lower.tail = TRUE)
+  ### one-sided greater
+  greater <- 1 - ppois(q = num.act-1, lambda = num.exp, lower.tail = TRUE)
+  ### pvalue
+  pcentral <- min(1, 2*min(less, greater))
+  expect_equal(a$pval, pcentral)
+  ## minlike
+  ### prob of getting what we got
+  prob <- dpois(x = num.act, lambda = num.exp) # 0.01341577
+  ### outcomes that are smaller AND less likely
+  probssmaller <- sapply(X = 0:(num.act-1), FUN = dpois, lambda = num.exp)
+  left <- sum(probssmaller[probssmaller <= prob])
+  ### outcomes that are larger AND less likely
+  #### know that Poisson distribution is unimodal -> check when get a prob to the right that is smaller
+  #### then sum all that are to the right & smaller than prob
+  i <- 1
+  while(dpois(x = num.act+i, lambda = num.exp) > prob) {
+    i <- i+1
+  }
+  probslarger <- 1 - ppois(q = num.act+i-1, lambda = num.exp, lower.tail = TRUE)
+  pminlike <- prob + sum(probssmaller) + probslarger
+  expect_equal(b$pval, pminlike)
+
 })
 
 test_that("multi_cutoff_to_fodr_vec() raises correct errors", {
 
+  skip_on_cran()
+
   p <- generate_param(3, 2, 3, sigma = 2, intercept = TRUE, seed = 42)
   d <- generate_data(parameters = p, n = 1000)$data
   gammas <- seq(0.01, 0.1, 0.01)
@@ -537,6 +635,8 @@ test_that("multi_cutoff_to_fodr_vec() raises correct errors", {
 
 test_that("multi_cutoff_to_fodr_vec() works correctly", {
 
+  skip_on_cran()
+
   p <- generate_param(3, 2, 3, sigma = 2, intercept = TRUE, seed = 42)
   d <- generate_data(parameters = p, n = 1000)$data
   gammas <- seq(0.01, 0.1, 0.01)
@@ -599,6 +699,8 @@ test_that("multi_cutoff_to_fodr_vec() works correctly", {
 
 test_that("sumtest() raises correct errors", {
 
+  skip_on_cran()
+
   expect_error(sumtest(robust2sls_object = 1, alpha = 0.05, iteration = 1,
                         one_sided = TRUE),
                "'robust2sls_object' must be a list of 'robust2sls' objects")
@@ -645,6 +747,8 @@ test_that("sumtest() raises correct errors", {
 
 test_that("sumtest() works correctly", {
 
+  skip_on_cran()
+
   p <- generate_param(3, 2, 3, sigma = 2, intercept = TRUE, seed = 42)
   d <- generate_data(parameters = p, n = 1000)$data
   gammas <- seq(0.01, 0.05, 0.01)
@@ -731,6 +835,8 @@ test_that("sumtest() works correctly", {
 
 test_that("suptest() raises correct error", {
 
+  skip_on_cran()
+
   expect_error(suptest(robust2sls_object = 1, alpha = 0.05, iteration = 1),
                "'robust2sls_object' must be a list of 'robust2sls' objects")
   expect_error(suptest(robust2sls_object = list(1, 2), alpha = 0.05,
@@ -771,6 +877,8 @@ test_that("suptest() raises correct error", {
 
 test_that("suptest() works correctly", {
 
+  skip_on_cran()
+
   p <- generate_param(3, 2, 3, sigma = 2, intercept = TRUE, seed = 42)
   d <- generate_data(parameters = p, n = 1000)$data
   gammas <- seq(0.01, 0.05, 0.01)
@@ -837,6 +945,8 @@ test_that("suptest() works correctly", {
 
 test_that("globaltest() raises correct errors", {
 
+  skip_on_cran()
+
   p <- generate_param(3, 2, 3, sigma = 2, intercept = TRUE, seed = 42)
   d <- generate_data(parameters = p, n = 1000)$data
   gammas <- seq(0.01, 0.05, 0.01)
@@ -863,6 +973,8 @@ test_that("globaltest() raises correct errors", {
 
 test_that("globaltest() works correctly", {
 
+  skip_on_cran()
+
   p <- generate_param(3, 2, 3, sigma = 2, intercept = TRUE, seed = 42)
   d <- generate_data(parameters = p, n = 1000)$data
   gammas <- seq(0.01, 0.05, 0.01)
@@ -889,7 +1001,7 @@ test_that("globaltest() works correctly", {
   expect_equal(a$global_alpha, 0.05)
 
   # count tests
-  tests <- counttest(models, alpha = 0.05, iteration = 0)
+  tests <- counttest(models, alpha = 0.05, iteration = 0, tsmethod = "minlike")
   a <- globaltest(tests = tests, global_alpha = 0.05)
   expect_snapshot_output(a)
   expect_equal(class(a), "list")
